@@ -19,17 +19,27 @@
 
 from .running.test_runner import main
 from .running.config import Seldom
-from .testdata.parameterized import data, data_class
-from .testdata.conversion import csv_to_list, excel_to_list, json_to_list
+from .testdata.parameterized import data, file_data, data_class
 from .webdriver import WebDriver
 from .case import TestCase
-from .driver import browser
+from .driver import ChromeConfig
+from .driver import FirefoxConfig
+from .driver import IEConfig
+from .driver import EdgeConfig
+from .driver import OperaConfig
+from .driver import SafariConfig
 from .skip import skip
 from .skip import skip_if
 from .skip import skip_unless
+from .skip import depend
+from .skip import if_depend
+from .logging.log import _print
+from .logging.log import info
+from .logging.log import error
+from .logging.log import debug
 
 __author__ = "bugmaster"
 
-__version__ = "1.5.0"
+__version__ = "1.8.0"
 
 __description__ = "WebUI automation testing framework based on Selenium."
