@@ -4,7 +4,8 @@ import sys
 import unittest
 import datetime
 from xml.sax import saxutils
-from seldom.running.config import RunResult
+from seldom.running.config import RunResult, Seldom
+from seldom.logging.exceptions import TestFixtureRunError
 
 """
 A TestRunner for use with the Python unit testing framework. It
@@ -178,7 +179,7 @@ class Template_mixin(object):
     <script src="https://cdn.bootcss.com/jquery/3.4.1/jquery.slim.min.js"></script>
     <script src="https://cdn.bootcss.com/popper.js/1.16.1/umd/popper.min.js"></script>
     <script src="http://apps.bdimg.com/libs/Chart.js/0.2.0/Chart.min.js"></script>
-    <link rel="stylesheet" href="http://img.itest.info/classic.css">
+    <link rel="stylesheet" href="http://img.itest.info/seldom.css">
 
     %(stylesheet)s
 </head>
@@ -381,7 +382,7 @@ function showOutput(id, name) {
 <style type="text/css" media="screen">
 body        { font-family: verdana, arial, helvetica, sans-serif; font-size: 80%; }
 table       { font-size: 100%; }
-pre         {  }
+pre         { overflow: initial; }
 /* -- heading ---------------------------------------------------------------------- */
 h1 {
 	font-size: 16pt;
@@ -766,7 +767,10 @@ class _TestResult(TestResult):
             sys.stderr = self.stderr0
             self.stdout0 = None
             self.stderr0 = None
-        return self.outputBuffer.getvalue()
+        try:
+            return self.outputBuffer.getvalue()
+        except AttributeError:
+            raise TestFixtureRunError("test setup/teardown run error.")
 
     def stopTest(self, test):
         # Usually one of addSuccess, addError or addFailure would have been called.
@@ -819,13 +823,11 @@ class _TestResult(TestResult):
         _, _exc_str = self.errors[-1]
         output = self.complete_output()
         self.result.append((2, test, output, _exc_str))
-        if not getattr(test, "driver", ""):
-            pass
-        else:
+        
+        if Seldom.driver is not None:
             try:
-                driver = getattr(test, "driver")
-                test.imgs.append(driver.get_screenshot_as_base64())
-            except BaseException:
+                test.imgs.append(Seldom.driver.get_screenshot_as_base64())
+            except BaseException as msg:
                 pass
         if self.verbosity > 1:
             sys.stderr.write('E  ')
@@ -841,12 +843,10 @@ class _TestResult(TestResult):
         _, _exc_str = self.failures[-1]
         output = self.complete_output()
         self.result.append((1, test, output, _exc_str))
-        if not getattr(test, "driver", ""):
-            pass
-        else:
+
+        if Seldom.driver is not None:
             try:
-                driver = getattr(test, "driver")
-                test.imgs.append(driver.get_screenshot_as_base64())
+                test.imgs.append(Seldom.driver.get_screenshot_as_base64())
             except BaseException:
                 pass
         if self.verbosity > 1:
