@@ -9,6 +9,7 @@ class Seldom:
     """
     driver = None
     timeout = None
+    debug = None
 
 
 class BrowserConfig:
@@ -16,8 +17,6 @@ class BrowserConfig:
     Define run browser config
     """
     name = None
-    driver_path = None
-    grid_url = None
     report_path = None
 
 
